@@ -8,7 +8,7 @@
 
 #define process_args() \
 	s = argv[i++]; \
-	int dirfd = atoi(s); \
+	int dirfd = at_get_fd(s); \
 	\
 	s = argv[i++]; \
 	char *file = s; \
@@ -18,7 +18,6 @@
 	sscanf(s, "%i", &mode); \
 	\
 	s = argv[i++]; \
-	int flags = 0; \
-	sscanf(s, "%i", &flags);
+	int flags = at_get_flags(s);
 
 #include "test-skel-0.c"
