@@ -8,19 +8,18 @@
 
 #define process_args() \
 	s = argv[i++]; \
-	int olddirfd = atoi(s); \
+	int olddirfd = at_get_fd(s); \
 	\
 	s = argv[i++]; \
 	char *oldpath = s; \
 	\
 	s = argv[i++]; \
-	int newdirfd = atoi(s); \
+	int newdirfd = at_get_fd(s); \
 	\
 	s = argv[i++]; \
 	char *newpath = s; \
 	\
 	s = argv[i++]; \
-	int flags = 0; \
-	sscanf(s, "%i", &flags);
+	int flags = at_get_flags(s);
 
 #include "test-skel-0.c"
